import {
  IEventParam, Metadata, IDictionary,
  Http, Loader,
} from '@zeedhi/core';
import {
  Button, Row, IComponentRender, Select, TextInput, Time, Text, AlertService, DialogService, Form,
} from '@zeedhi/common';

export class FuncionariosController {
  private dataCards: IDictionary = [];

  private modalController = Loader.getInstance('ModalController');

  public labelName: string = 'Salvar';

  public componentButtonDelete: IDictionary = {};

  private events: any = {
    click: () => {
      this.deleteEmployee();
      DialogService.hide();
    },
  };

  private eventsX: any = {
    click: () => {
      DialogService.hide();
    },
  };

  private dataFilter: IDictionary = {
    InputName: '',
    SelectOffice: '',
    SelectShift: '',
  };

  public dataModel: IDictionary = {
    nameEmplyoee: '',
    nrfuncionario: '',
    nrrelacfunchab: '',
    officeEmployee: '',
    shift: '',
    task: '',
  };

  public tasksModel: IDictionary = {
    typeTask: '',
    localTask: '',
    timeTask: '',
  };

  public dataSelects: IDictionary = {
    SelectOffice: [],
    SelectShift: [],
    SelectTask: [],
  };

  public dataInputs: IDictionary = {
    inputRG: '',
    inputCPF: '',
    inputEmail: '',
    inputTelephone: '',
  };

  public filter() {
    this.dataFilter.InputName = this.getName;
    this.dataFilter.SelectOffice = this.getOffice;
    this.dataFilter.SelectShift = this.getShift;
    this.createCards();
    this.initializeCards();
  }

  public setData(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  get getName() {
    return Metadata.getInstance<TextInput>('InputName').value || '';
  }

  get getOffice() {
    return Metadata.getInstance<Select>('SelectOffice').value || '';
  }

  get getShift() {
    return Metadata.getInstance<Select>('SelectShift').value || '';
  }

  public setDataShift(cod: number) {
    let nome: string = '';
    switch (cod) {
      case 1:
        nome = 'Matutino';
        break;
      case 2:
        nome = 'Vespertino';
        break;
      default:
        nome = 'Noturno';
        break;
    }
    return {
      name: nome,
      value: cod,
    };
  }

  public setOffice() {
    Http.get('/getOffice').then((response: IDictionary) => {
      response.data.dataset.office.forEach((dados: IDictionary) => {
        this.dataSelects.SelectOffice.push(this.setData(dados.NMHABILIDADE, dados.NRHABILIDADE));
      });
    });
  }

  public setTasks() {
    Http.get('/getTasks').then((response: IDictionary) => {
      response.data.dataset.tasks.forEach((dados: IDictionary) => {
        this.dataSelects.SelectTask.push(this.setData(dados.NMCHECKLIST, dados.NRCHECKLIST));
      });
    });
  }

  public initializeData() {
    this.setOffice();
    this.setTasks();
    this.createCards();
  }

  public getDataInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  public change({ component }: IEventParam<Button>) { // instancia quando o componente não é fixo e é criado pelo controller
    const cardEmplyoee = this.getParent(component, 'ZdCollapseCard');
    if (component.label === 'Ver mais') {
      cardEmplyoee.expanded = true;
      const nrfuncionario1 = cardEmplyoee.header[0].children[0].children[0].children[2];
      component.label = 'Ver menos';
      this.listTasks(cardEmplyoee.children[1].tabs[1].children[0], nrfuncionario1.text);
    } else {
      cardEmplyoee.expanded = false;
      component.label = 'Ver mais';
      cardEmplyoee.children[1].tabs[1].children[0].items.splice(0, cardEmplyoee.children[1].tabs[1].children[0].items.length);
    }
  }

  public getParent(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  public createCards() {
    this.dataCards = [];
    Http.post('/getDataBaseCards', this.dataFilter).then((response) => {
      response.data.dataset.dataBaseCards.forEach((database: IDictionary) => {
        this.dataModel.nameEmplyoee = database.NMFUNCIONARIO;
        this.dataModel.officeEmployee = database.NMHABILIDADE;
        this.dataModel.shift = database.IDTURNOFUNCIONARIO;
        this.dataModel.nrfuncionario = database.NRFUNCIONARIO;
        this.dataModel.nrrelacfunchab = database.NRRELACFUNCHAB;
        this.dataInputs.inputRG = database.NRRGFUNCIONARIO;
        this.dataInputs.inputCPF = database.NRCPFFUNCIONARIO;
        this.dataInputs.inputEmail = database.DSEMAILFUNCIONARIO;
        this.dataInputs.inputTelephone = database.NRTELFUNCIONARIO;
        this.dataCards.push(this.modelCards(this.dataModel, this.dataInputs));
      });
    });
  }

  public initializeCards() {
    Metadata.getInstance<Row>('RowCards').children = this.dataCards as IComponentRender[];
  }

  public listTasks(list: any, nrfuncionario: number) {
    const employeesTasks = {
      NRFUNCIONARIO: nrfuncionario,
    };
    Http.post('/getDataBaseTaskCards', { employeesTasks }).then((response) => {
      response.data.dataset.dataBaseCards.forEach((database: IDictionary) => {
        this.tasksModel.typeTask = database.NMTAREFA;
        this.tasksModel.localTask = database.NMLOCALCLIENTE;
        this.tasksModel.timeTask = database.DTHORATAREFAAGEND;
        list.items.push(this.modelList(this.tasksModel));
      });
    });
  }

  public deleteEmployee() {
    const component = this.componentButtonDelete;
    const cardEmplyoee = this.getParent(component as IComponentRender, 'ZdCollapseCard');
    const nrfuncionario = cardEmplyoee.header[0].children[0].children[0].children[2];
    const nrrelacfunchab = cardEmplyoee.header[0].children[0].children[0].children[3];
    const deleteEmployee = {
      NRFUNCIONARIO: nrfuncionario.text,
      NRRELACFUNCHAB: nrrelacfunchab.text,
      NRORG: '20',
    };
    Http.post('/postDeleteEmployee', { deleteEmployee });
    AlertService.show({
      name: 'Novo funcionário',
      text: 'Funcionário deletado com sucesso!!!',
      timeout: 2000,
      top: true,
      left: false,
      color: 'info',
    });
    window.location.reload();
  }

  public showCancelDelete({ component }: IEventParam<Button>) {
    this.componentButtonDelete = component;
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir funcionário',
      text: 'Todas as informações deste funcionário serão perdidas. Tem certeza que deseja excluir este funcionário?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'YES',
          events: this.events,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'NO',
          events: this.eventsX,
        },
      ],
    });
  }

  public toEdit({ component }: IEventParam<Button>) {
    const cardEmplyoee = this.getParent(component, 'ZdCollapseCard');
    const nrfuncionario = cardEmplyoee.header[0].children[0].children[0].children[2];
    const nrrelacfunchab = cardEmplyoee.header[0].children[0].children[0].children[3];
    const getInfo = {
      NRFUNCIONARIO: nrfuncionario.text,
      NRRELACFUNCHAB: nrrelacfunchab.text,
      NRORG: '20',
    };
    this.modalController.openmodalFuncionario();
    Http.post('/getInfoEmployee', { getInfo }).then((response) => {
      response.data.dataset.getInfomation.forEach((database: IDictionary) => {
        Metadata.getInstance<Text>('upNrFuncionario').text = database.NRFUNCIONARIO;
        Metadata.getInstance<Text>('upNrrelacfunchab').text = database.NRRELACFUNCHAB;
        Metadata.getInstance<TextInput>('nameEmployee').value = database.NMFUNCIONARIO;
        Metadata.getInstance<TextInput>('RGEmployee').value = database.NRRGFUNCIONARIO;
        Metadata.getInstance<TextInput>('CPFEmployee').value = database.NRCPFFUNCIONARIO;
        Metadata.getInstance<TextInput>('insEmail').value = database.DSEMAILFUNCIONARIO;
        Metadata.getInstance<TextInput>('insTelephone').value = database.NRTELFUNCIONARIO;
        Metadata.getInstance<Time>('timeStart').value = database.HRENTRADAPADR;
        Metadata.getInstance<Time>('timeEnd').value = database.HRSAIDAPADR;
        Metadata.getInstance<Select>('insOccupation').setValue(database.NRHABILIDADE);
        Metadata.getInstance<Select>('selectShift').setValue(database.IDTURNOFUNCIONARIO);
      });
    });
    this.changeName();
  }

  public modelCards(modeloDados: IDictionary, dataInputs: IDictionary): IComponentRender {
    return {
      name: 'CardsEmployees',
      component: 'ZdCollapseCard',
      readonly: true,
      hover: true,
      style: {
        'margin-top': '10px',
        'margin-left': '3px',
        'margin-right': '3px',
      },
      expandButton: {
        name: 'ButtonViewMore',
        component: 'ZdButton',
        label: 'Ver mais',
        flat: true,
        color: 'info',
        events: {
          click: '{{FuncionariosController.change}}',
        },
      },
      header: [
        {
          name: 'HeaderCard',
          component: 'ZdRow',
          children: [
            {
              name: 'ColHeader',
              component: 'ZdCol',
              cols: 10,
              children: [
                {
                  name: 'RowCardHeader',
                  component: 'ZdRow',
                  cssClass: 'zd-mt-2',
                  children: [
                    {
                      name: 'InputName',
                      component: 'ZdText',
                      text: modeloDados.nameEmplyoee,
                      style: {
                        color: '#2196f3',
                        'font-weight': 'bold',
                        'margin-right': '5px',
                        'margin-left': '15px',
                      },
                    },
                    {
                      name: 'OfficeShift',
                      component: 'ZdText',
                      text: `${modeloDados.officeEmployee} | ${modeloDados.shift}`,
                      style: {
                        color: '#6c757d',
                      },
                    },
                    {
                      name: '',
                      component: 'ZdText',
                      text: modeloDados.nrfuncionario,
                      isVisible: false,
                    },
                    {
                      name: '',
                      component: 'ZdText',
                      text: modeloDados.nrrelacfunchab,
                      isVisible: false,
                    },
                  ],
                },
              ],
            },
            {
              name: 'colButtons',
              component: 'ZdCol',
              cssClass: 'text-end',
              cols: 2,
              children: [
                {
                  name: 'buttonEdit',
                  component: 'ZdButton',
                  icon: true,
                  iconName: 'mdi-pencil',
                  events: {
                    click: '{{FuncionariosController.toEdit}}',
                  },
                },
                {
                  name: 'buttonDelete',
                  component: 'ZdButton',
                  icon: true,
                  iconName: 'mdi-delete',
                  events: {
                    click: '{{FuncionariosController.showCancelDelete}}',
                  },
                },
              ],
            },
          ],
        },
      ],
      children: [
        {
          name: 'basicUsageDivider',
          component: 'ZdDivider',
        },
        {
          name: 'tabsCards',
          component: 'ZdTabs',
          tabs: [
            {
              name: 'tabDados',
              tabTitle: 'Informações Pessoais',
              children: [
                {
                  name: '',
                  component: 'ZdRow',
                  children: [
                    {
                      name: 'inputRG',
                      component: 'ZdTextInput',
                      readonly: true,
                      label: 'RG',
                      mask: '##.###.###',
                      value: dataInputs.inputRG,
                      style: {
                        'margin-left': '20px',
                        'margin-right': '60px',
                      },
                    },
                    {
                      name: 'inputCPF',
                      component: 'ZdTextInput',
                      readonly: true,
                      label: 'CPF',
                      mask: '###.###.###-##',
                      value: dataInputs.inputCPF,
                      style: {
                        'margin-left': '20px',
                        'margin-right': '60px',
                      },
                    },
                  ],
                },
                {
                  name: '',
                  component: 'ZdRow',
                  children: [
                    {
                      name: 'inputEmail',
                      component: 'ZdTextInput',
                      readonly: true,
                      label: 'E-mail',
                      value: dataInputs.inputEmail,
                      style: {
                        'margin-left': '20px',
                        'margin-right': '60px',
                      },
                    },
                    {
                      name: 'inputTelephone',
                      component: 'ZdTextInput',
                      readonly: true,
                      label: 'Telefone',
                      value: dataInputs.inputTelephone,
                      style: {
                        'margin-left': '20px',
                        'margin-right': '60px',
                      },
                    },
                  ],
                },
              ],
            },
            {
              name: 'tabTasks',
              tabTitle: 'Tarefas do Dia',
              children: [
                {
                  name: 'ListTasks',
                  component: 'ZdList',
                  divided: true,
                  dense: true,
                  disabled: true,
                  items: [],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public modelList(modeloTarefas : IDictionary) {
    return {
      name: 'ItemList',
      component: 'ZdListItem',
      prependIcon: 'mdi-square-small',
      title: `${modeloTarefas.typeTask} - ${modeloTarefas.localTask} | Horário: ${modeloTarefas.timeTask}`,
    };
  }
  //
  // Métodos modal

  public saveDados() {
    const validateFormTime = Metadata.getInstance<Form>('formTime').validate();
    const validateDataBase = Metadata.getInstance<Form>('formDataBase').validate();
    //
    AlertService.show({
      name: '',
      text: 'Preencha todos os campos',
      timeout: 2000,
      top: true,
      left: false,
      color: 'info',
    });
    if (validateDataBase && validateFormTime) {
      if (Metadata.getInstance<Button>('saveButton').label === 'Salvar') {
        this.insertNewEmployee();
      } else {
        this.updateEmployee();
      }
      this.modalController.closemodalFuncionario();
      AlertService.show({
        name: 'Novo funcionário',
        text: 'Novo funcionário adicionado com sucesso!!!',
        timeout: 2000,
        top: true,
        left: false,
        color: 'info',
      });
      this.labelName = 'Salvar';
      window.location.reload();
    }
  }

  public insertNewEmployee() {
    const NewEmployees = {
      NMFUNCIONARIO: Metadata.getInstance<TextInput>('nameEmployee').value,
      NRRGFUNCIONARIO: Metadata.getInstance<TextInput>('RGEmployee').value,
      NRCPFFUNCIONARIO: Metadata.getInstance<TextInput>('CPFEmployee').value,
      DSEMAILFUNCIONARIO: Metadata.getInstance<TextInput>('insEmail').value,
      NRTELFUNCIONARIO: Metadata.getInstance<TextInput>('insTelephone').value,
      HRENTRADAPADR: Metadata.getInstance<Time>('timeStart').value,
      HRSAIDAPADR: Metadata.getInstance<Time>('timeEnd').value,
      IDTURNOFUNCIONARIO: Metadata.getInstance<Select>('selectShift').value,
      NRHABILIDADE: Metadata.getInstance<Select>('insOccupation').value,
      NRORG: '20',
      OPERADOR: '000000004280',
    };
    Http.post('/postNewEmployee', { NewEmployees });
  }

  public updateEmployee() {
    const updateEmployee = {
      NRFUNCIONARIO: Metadata.getInstance<Text>('upNrFuncionario').text,
      NRRELACFUNCHAB: Metadata.getInstance<Text>('upNrrelacfunchab').text,
      NMFUNCIONARIO: Metadata.getInstance<TextInput>('nameEmployee').value,
      NRRGFUNCIONARIO: Metadata.getInstance<TextInput>('RGEmployee').value,
      NRCPFFUNCIONARIO: Metadata.getInstance<TextInput>('CPFEmployee').value,
      DSEMAILFUNCIONARIO: Metadata.getInstance<TextInput>('insEmail').value,
      NRTELFUNCIONARIO: Metadata.getInstance<TextInput>('insTelephone').value,
      HRENTRADAPADR: Metadata.getInstance<Time>('timeStart').value,
      HRSAIDAPADR: Metadata.getInstance<Time>('timeEnd').value,
      NRHABILIDADE: Metadata.getInstance<Select>('insOccupation').value,
      IDTURNOFUNCIONARIO: Metadata.getInstance<Select>('selectShift').value,
      NRORG: '20',
      OPERADOR: '000000004280',
    };
    Http.post('/postUpdateEmployee', { updateEmployee });
  }

  public changeName() {
    this.labelName = 'Atualizar';
  }

  public closeModal() {
    this.modalController.closemodalFuncionario();
    this.labelName = 'Salvar';
  }
}
