import {
  Metadata, IEventParam, IDictionary, Http,
} from '@zeedhi/core';
import {
  Form, Chip, Text, Select, SelectMultiple, // Row, // TextInput, Row, DialogService,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class NovoChecklistController {
  public dadosTarefa: IDictionary = {
    selectLocal: [],
    selectItens: [],
    selectTagTarefas: [],
  };

  public setDados(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public setLocal() {
    Http.get('/getLocalChk').then((response: IDictionary) => {
      response.data.dataset.local.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectLocal.push(this.setDados(dados.NMTIPOLOCALCLI, dados.NRTAREFACHK));
      });
    });
  }

  public setItens() {
    Http.get('/filtroItens').then((response: IDictionary) => {
      response.data.dataset.filtro.forEach((dados: IDictionary) => {
        this.dadosTarefa.selectItens.push(this.setDados(dados.NMATIVIDADE, dados.NRATIVIDADE));
      });
    });
  }

  public setTag() {
    Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = [];
    const tarefa = Metadata.getInstance<SelectMultiple>('selectTagTarefas');
    tarefa.datasource.data = [];
    tarefa.setValue([]);
    Http.post('/filtroTags', Metadata.getInstance<SelectMultiple>('selectItens').value).then((response: IDictionary) => {
      response.data.dataset.filtroTags.forEach((dados: IDictionary) => {
        tarefa.datasource.data.push({
          name: dados.NMTAREFA,
          value: dados.NRTAREFA,
          parent: dados.NRATIVIDADE,
          parentName: dados.NMATIVIDADE,
        });
      });
    });
  }

  get descricaoNovoChecklist() {
    const descricao = Metadata.getInstance<Text>('descriçaoChecklist');
    return descricao;
  }

  get itensNovoChecklist() {
    const itens = Metadata.getInstance<Text>('itensNovoChecklist');
    return itens;
  }

  public getDadosInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dadosTarefa[component.name];
  }

  get tarefaCheckList() {
    const tarefa = Metadata.getInstance<Select>('SelectTarefas');
    return tarefa.value;
  }

  public getParent(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  public botaoCriarTarefa() {
    const form = Metadata.getInstance<Form>('formCadastroNovoChecklist');
    if (form.validate()) {
      form.resetValidation();
    }
  }

  public selecionaGrupoCriterio({ component }: IEventParam<Chip>) {
    const selectObject = this.getParent(component, 'ZdChip');
    if (selectObject.click) {
      // const itensClicados = component.label;
      // Metadata.getInstance<Text>('tituloNovaCheckList').text = itensClicados;
      // this.listCriterios = itensClicados;
      // Metadata.getInstance<Text>('tituloNovaCheckList').text = this.listCriterios;
      if (component.outlined === true) {
        component.outlined = false;
      } else {
        component.outlined = true;
      }
    }
  }

  public inicializarDados() {
    this.setLocal();
    this.setItens();
  }

  public modeloCardCriterio(titulo: string, id: string) {
    return {
      name: '',
      component: 'ZdRow',
      cssClass: 'zd-pb-5',
      children: [
        {
          name: '',
          component: 'ZdCol',
          children: [
            {
              name: '',
              component: 'ZdRow',
              children: [
                {
                  name: '',
                  component: 'ZdCard',
                  children: [
                    {
                      name: '',
                      component: 'ZdText',
                      text: `${titulo}`,
                    },
                    {
                      name: id,
                      component: 'ZdRow',
                      events: {
                        onCreated: '{{NovoChecklistController.teste}}',
                      },
                      children: [],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public modeloTag(tarefas: string) {
    return ({
      name: 'chipResult',
      component: 'ZdChip',
      label: `${tarefas}`,
      cssClass: 'zd-my-2',
      color: 'info',
    });
  }

  public dadosData: IDictionary = [];

  public renderizaTarefas({ component }: IEventParam<SelectMultiple>) {
    Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = [];
    this.dadosData = [];
    if (component.value.length) {
      Http.post('/getGrupo', component.value).then((response: IDictionary) => {
        response.data.dataset.grupos.forEach((dados: IDictionary) => {
          this.dadosData.push({
            DSTAGSTAREFA: dados.DSTAGSTAREFA,
            list: dados.list,
          });
        });
        // const verificacao = this.dadosData.filter((dados: IDictionary, pos: number) => this.dadosData.findIndex((valor: IDictionary) => valor.DSTAGSTAREFA === dados.DSTAGSTAREFA) === pos);
        Metadata.getInstance<IterableComponentRender>('iterable_parent').datasource.data = this.dadosData as IDictionary[];
      });
    }
  }

  public consoleLog(variavel: string) {
    const titulo = Metadata.getInstance<Text>('tituloNovaCheckList');
    titulo.text = variavel;
  }

  // public insertNovoChecklistBase() {
  //   const selectObjectForm = Metadata.getInstance<Form>('formCadastroNovoChecklist');
  //   if (selectObjectForm.validate()) {
  //     const arrayDados = {
  //       tipoLocal: Metadata.getInstance<Select>('selectLocal').value,
  //       nomeCheckList: Metadata.getInstance<TextInput>('nomeCheckList').value,
  //       descricaoChecklist: Metadata.getInstance<TextInput>('descricaoChecklist').value,
  //       itensChecklist: Metadata.getInstance<SelectMultiple>('selectItens').value,
  //       selectTagTarefas: Metadata.getInstance<SelectMultiple>('selectTagTarefas').value,
  //     };
  //     this.consoleLog(arrayDados);
  //   } else {
  //     DialogService.show({
  //       name: 'error-dialog',
  //       type: 'error',
  //       text: 'Favor Preencher os campos Solicitados.',
  //     });
  //   }
  // }

  public getDadosCard({ component }: IEventParam<IterableComponentRender>) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('iterable_parent');
    const id = component.name.split('_').pop();
    const parentIndex = parentIterable.datasource.data.findIndex((row: IDictionary) => row.DSTAGSTAREFA === id);
    const destructuring = parentIterable.datasource.data[parentIndex].list;
    component.datasource.data = destructuring;
  }

  public getDadosCriterios({ component }: IEventParam<IterableComponentRender>) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('iterable_parent');
    const nome = component.name.split('_');
    const idFilho = nome.pop();
    const idPai = nome.pop();
    const parentIndex = parentIterable.datasource.data.findIndex((row: IDictionary) => row.DSTAGSTAREFA === idPai);
    const filhoIndex = parentIterable.datasource.data[parentIndex].list.findIndex((row: IDictionary) => row.NRATIVIDADE === Number(idFilho));
    const dadoscard = parentIterable.datasource.data[parentIndex].list[filhoIndex].list;
    component.datasource.data = dadoscard;
  }
}
