import {
  Metadata, Http, IDictionary,
  IEventParam,
} from '@zeedhi/core';
import {
  Select, Text, IComponentRender, Row, Date,
} from '@zeedhi/common';

export class TarefaController {
  public dateValue: string = '';

  public dadosFiltro: IDictionary = {
    selectTipoTarefa: [],
    selectTipoLocal: [],
    selectAndar: [],
    selectStatus: [],
    selectFuncionario: [],
  };

  public list2: IDictionary = [];

  public dadosCard: IComponentRender[] = [];

  public setDados(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public dataFilter: string = '';

  public setCards(variavel: IDictionary = {}) {
    this.dadosCard = [];
    Http.post('/getCard', { variavel1: variavel }).then((response: IDictionary) => {
      response.data.dataset.cards.forEach((dados: IDictionary) => {
        this.dadosFiltro.selectTipoTarefa.push(this.setDados(dados.NMCHECKLISTAGEND, dados.NRCHECKLISTAGEND));
        this.dadosFiltro.selectTipoLocal.push(this.setDados(dados.NMLOCALCLIENTE, dados.NRLOCALCLIENTE));
        this.dadosFiltro.selectTipoLocal.push(this.setDados(dados.NMLOCALCLIENTE2, dados.NRLOCALCLIENTEPAI));
        this.dadosFiltro.selectAndar.push(this.setDados(dados.NMCHECKLIST, dados.NRCHECKLIST));
        this.dadosFiltro.selectStatus.push(this.setDados(dados.NMIDSITUACAOAGEND, dados.IDSITUACAOAGEND));
        this.dadosFiltro.selectFuncionario.push(this.setDados(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
        let dataHora = dados.DTHORAAGEND.split(' ');
        dataHora = `${dataHora[0].split('-')[2]}-${dataHora[0].split('-')[1]}-${dataHora[0].split('-')[0]} ${dataHora[1].substr(0, 5)}`;
        this.dadosCard.push(this.dadosTarefa({
          nome: dados.NMCHECKLISTAGEND,
          local: dados.NMLOCAL,
          horario: dataHora,
          funcionario: (dados.NMFUNCIONARIO.length > 22) ? `${dados.NMFUNCIONARIO.substr(0, 40)}...` : dados.NMFUNCIONARIO,
          statu: dados.NMIDSITUACAOAGEND,
        }));
      });
    });
  }

  public filtroCards() {
    const data = Metadata.getInstance<Date>('selectData').value;
    return {
      'A.NRCHECKLISTAGEND': Metadata.getInstance<Select>('selectTipoTarefa').value,
      'C.NRLOCALCLIENTE': Metadata.getInstance<Select>('selectTipoLocal').value,
      'A.NRCHECKLIST': Metadata.getInstance<Select>('selectAndar').value,
      'A.IDSITUACAOAGEND': Metadata.getInstance<Select>('selectStatus').value,
      'A.NRFUNCIONARIO': Metadata.getInstance<Select>('selectFuncionario').value,
      'TO_CHAR(A.DTHORAAGEND,\'DD/MM/YYYY\')': ((data != null) ? `TO_CHAR('${data}')` : null),
    };
  }

  public filtrar() {
    this.setCards(this.filtroCards());
    this.getCards();
  }

  public getCards() {
    Metadata.getInstance<Row>('Cards_de_Tarefas').children = this.dadosCard as IComponentRender[];
  }

  public inicializarDados() {
    this.setCards();
  }

  public dadosTarefa(variavel: IDictionary) {
    return {
      name: 'ColunaCard',
      component: 'ZdCol',
      children: [
        {
          name: 'Cards',
          component: 'ZdCard',
          children: [
            {
              name: 'linhaInterna',
              component: 'ZdRow',
              justify: 'space-between',
              children: [
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: variavel.nome,
                          style: {
                            color: '#2196f3',
                            'font-weight': 'bold',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: 'mdi-map-marker-outline',
                          tag: 'v-icon',
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: variavel.local,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: 'mdi-clock-check-outline',
                          tag: 'v-icon',
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: variavel.horario,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '3',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: 'mdi-account-outline',
                          tag: 'v-icon',
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: variavel.funcionario,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      cssClass: 'zd-justify-end',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: variavel.statu,
                          events: {
                            onBeforeMount: '{{TarefaController.styleColor}}',
                          },
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  public getParent(component: IEventParam<any>, nameComponent: string): any {
    if (component.component === nameComponent) {
      return component;
    }
    if (component.parent) {
      return this.getParent(component.parent, nameComponent);
    }
    return null;
  }

  public add(nome: string, valor: number) {
    return {
      name: nome,
      value: valor,
    };
  }

  public filtroTipoTarefa({ component }: IEventParam<Select>) {
    component.datasource.data = this.dadosFiltro[component.name];
  }

  public mudaTitulo(dig: string) {
    const selectObject = Metadata.getInstance<Text>('TituloPrincipal');
    selectObject.text = dig;
  }

  public styleColor({ component }: IEventParam<Text>) {
    if (component.text === 'Pendente') {
      component.text = '<span style="color: #E9C46A;">Pendente</span>';
    } else if (component.text === 'Em Andamento') {
      component.text = '<span style="color: #E9C46A;">Em Andamento</span>';
    } else if (component.text === 'Concluido') {
      component.text = '<span style="color: #E9C46A;">Concluido</span>';
    } else {
      component.text = '<span style="color: #E9C46A;">Cancelado</span>';
    }
  }
}
